/*************************************************************
*	PROGETTO LABIRINTO
*
* nome file: <engine.c>
*
* Autori:
* Thomas Carrabs
* Roberto Pisoni
*
* Progetto realizzato per l'esame di Geometria Computazionale
* e di Informatica Grafica presso il Dipartimento di Scienze
* dell'Informazione dell'universit Statale di Milano
*
* Tutto il codice  distribuito sotto licenza Gnu/GPL
* http://www.gnu.org/copyleft/gpl.html
* una copia della licenza si trova nella cartella License del
* Progetto
*
*************************************************************/

#include "engine.h"
#if defined(WIN32)
#  include <GL\glext.h>
#endif

extern struct maze maze;
extern GLuint mappa;
extern int *soluz_lab;

FILE *fp;

GLuint IlLabirinto;
GLuint LaMappa;
GLuint LaSoluzione;
GLuint IlPavimento;
GLuint IlSoffitto;


/*---------------------------VARIABILI TEXTURE-------------------------------*/
char texture_muro[]="texture/wall.bmp";		//nome texture file per il muro
char texture_pav[]="texture/floor.bmp";		//nome texture file per il pavimento
char texture_roof[]="texture/roof.bmp";		//nome texture file per il soffitto
char texture_exit[]="texture/door.bmp";		//nome texture file per la botola di uscita
char texture_pill[]="texture/pillar.bmp";  //nome texture file per i pilastri
char texture_entr[]="texture/entrata.bmp";  //nome texture file per i pilastri
GLuint texture_id[MAX_TEXTURES];		//array per l'id delle texture
//ARRAY ELEMENTI TEXTURE
GLubyte *texels_muro,*texels_pav, *texels_roof,*texels_exit,*texels_pill,*texels_entr;
GLfloat s,t;						//coordinate texture
/*--------------------------END VARIABILI TEXTURE----------------------------*/

short int init_Texture(){
/*
 * Questa funzione verifica l'esistenza e carica le texture
 * oltre a creare una NORMAL CUBE MAP
 */
	glGenTextures (MAX_TEXTURES, texture_id);  //genera 4 nuovi nomi di texture
	glTexCoord2f(s,t);				//setta le coordinate texture. s e t variano tra 0 e 1, r=0 e q=1.
	//***TEXTURE MURO***
	if((texels_muro=calloc(128*128*3,sizeof(GLubyte)))==NULL){
		printf("\nImpossibile allocare memoria per la bitmap.");
		return 0;
	}
	if(!caricaTexture(texture_muro,texels_muro))
		exit(0);
	/*crea un texture object con id texture_id[0] di dimensione texture 2D. Le funzioni sottostanti fino all'if
	influenzano il texture object*/
	glBindTexture (GL_TEXTURE_2D, texture_id[0]);
	//definisce la texture bidimensionale di dimensione 128x128
	glTexImage2D(GL_TEXTURE_2D,0,3,128,128,0,GL_RGB,GL_UNSIGNED_BYTE, texels_muro);
	//costruisce le mipmaps fino a 1x1 per ovviare al problema di minification (texel + piccola di un pixel)
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,128,128,GL_RGB,GL_UNSIGNED_BYTE, texels_muro);
	//settaggio parametri di controllo della texture
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);	//texture ripetuta sull'asse s
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT); //texture ripetuta sull'asse t
	//setta le funzioni di texture magnification e minification: sceglie 2 mipmaps + o - grandi al pixel
	//texturizzato e fa una media pesata delle 4 texels vicine al centro del pixel.
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	//setta la funzione di texturing. In questo caso GL_DECAL: in pratica uso i colori della texture
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	
	//***TEXTURE PAVIMENTO***
	if((texels_pav=calloc(128*128*3,sizeof(GLubyte)))==NULL){
		printf("\nImpossibile allocare memoria per la bitmap.");
		return 0;
	}
	if(!caricaTexture(texture_pav,texels_pav))
		exit(0);
	glBindTexture (GL_TEXTURE_2D, texture_id[1]);
	glTexImage2D(GL_TEXTURE_2D,0,3,128,128,0,GL_RGB,GL_UNSIGNED_BYTE, texels_pav);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,128,128,GL_RGB,GL_UNSIGNED_BYTE, texels_pav);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	
	//***TEXTURE SOFFITTO***
	if((texels_roof=calloc(128*128*3,sizeof(GLubyte)))==NULL){
		printf("\nImpossibile allocare memoria per la bitmap.");
		return 0;
	}
	if(!caricaTexture(texture_roof,texels_roof))
		exit(0);
	glBindTexture (GL_TEXTURE_2D, texture_id[2]);
	glTexImage2D(GL_TEXTURE_2D,0,3,128,128,0,GL_RGB,GL_UNSIGNED_BYTE, texels_roof);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,128,128,GL_RGB,GL_UNSIGNED_BYTE, texels_roof);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	//***TEXTURE BOTOLA USCITA***
	if((texels_exit=calloc(128*128*3,sizeof(GLubyte)))==NULL){
		printf("\nImpossibile allocare memoria per la bitmap.");
		return 0;
	}
	if(!caricaTexture(texture_exit,texels_exit))
		exit(0);
	glBindTexture (GL_TEXTURE_2D, texture_id[3]);
	glTexImage2D(GL_TEXTURE_2D,0,3,128,128,0,GL_RGB,GL_UNSIGNED_BYTE, texels_exit);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,128,128,GL_RGB,GL_UNSIGNED_BYTE, texels_exit);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	
	//***TEXTURE PILASTRI***
	if((texels_pill=calloc(128*128*3,sizeof(GLubyte)))==NULL){
		printf("\nImpossibile allocare memoria per la bitmap.");
		return 0;
	}
	if(!caricaTexture(texture_pill,texels_pill))
		exit(0);
	glBindTexture (GL_TEXTURE_2D, texture_id[4]);
	glTexImage2D(GL_TEXTURE_2D,0,3,128,128,0,GL_RGB,GL_UNSIGNED_BYTE, texels_pill);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,128,128,GL_RGB,GL_UNSIGNED_BYTE, texels_pill);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	//***TEXTURE PORTA INGRESSO***
	if((texels_entr=calloc(128*128*3,sizeof(GLubyte)))==NULL){
		printf("\nImpossibile allocare memoria per la bitmap.");
		return 0;
	}
	if(!caricaTexture(texture_entr,texels_entr))
		exit(0);
	glBindTexture (GL_TEXTURE_2D, texture_id[5]);
	glTexImage2D(GL_TEXTURE_2D,0,3,128,128,0,GL_RGB,GL_UNSIGNED_BYTE, texels_entr);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,128,128,GL_RGB,GL_UNSIGNED_BYTE, texels_entr);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	return 1;
}

void init_Pavimento(){
/*
 * Crea una display List per il pavimento
 */
	int i, x, z;
	
	
	IlPavimento = glGenLists(1);
	glNewList(IlPavimento, GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D, texture_id[1]);	//attivo il texture object in texture_id[1]
	for(i = 0; i <maze.m*maze.n; i++){
		if(i==maze.uscita->id){							 //se i  = all'indice della cella di uscita...
			glBindTexture(GL_TEXTURE_2D, texture_id[3]); //cambio la texture: uso quella della botola
			x = i%maze.n * 10;
			z = i/maze.n * 10;		
			glNormal3f(0.0, 1.0, 0.0);
			draw_Pannel(10, 10, dMeshH, dMeshV, x, z, 0);
			glBindTexture(GL_TEXTURE_2D, texture_id[1]); //ripristino la texture del pavimento
		}
		else{
		x = i%maze.n * 10;
		z = i/maze.n * 10;		
		glNormal3f(0.0, 1.0, 0.0);
		draw_Pannel(10, 10, dMeshH, dMeshV, x, z, 0);
		}
	}
	glEndList();

	
}

void init_Soffitto(){
/*
 * Crea una display list per il soffitto
 */
	int i, x, z;
	
	
	IlSoffitto = glGenLists(1);
	glNewList(IlSoffitto, GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D, texture_id[2]);	//attivo il texture object in texture_id[1]
	for(i = 0; i <maze.m*maze.n; i++){
		if(i==maze.entrata->id){							 //se i  = all'indice della cella di ingresso...
			glBindTexture(GL_TEXTURE_2D, texture_id[5]); //cambio la texture: uso quella della porta di ingresso
			x = i%maze.n * 10;
			z = i/maze.n * 10;		
			glNormal3f(0.0, -1.0, 0.0);
			draw_Pannel(10, 10, dMeshH, dMeshV, x, z, 10);
			glBindTexture(GL_TEXTURE_2D, texture_id[2]); //ripristino la texture del soffitto
		}
		else{
		x = i%maze.n * 10;
		z = i/maze.n * 10;		
		glNormal3f(0.0, -1.0, 0.0);
		draw_Pannel(10, 10, dMeshH, dMeshV, x, z, 10);
		}
	}
	glEndList();

}

void init_Soluzione(){
/*
 * Crea una display list per la soluzione del labirinto
 * da visualizzare nella mappa 2D
 */
	int i, x, y;

	LaSoluzione = glGenLists(1);
	glNewList(LaSoluzione, GL_COMPILE);
	glColor3f(0.0, 0.0, 1.0);
	
	i = maze.uscita->id;
	while(soluz_lab[i] != -1){
		x = i%maze.n * 10;
		y = i/maze.n * 10;
		glPushMatrix();
		glTranslatef(x,y,0.0);
		glBegin(GL_QUADS);
			glVertex2i(0.0, 0.0);
			glVertex2i(10.0, 0.0);
			glVertex2i(10.0, 10.0);
			glVertex2i(0.0, 10.0);
		glEnd();
		glPopMatrix();
		i = soluz_lab[i];
	}
	glEndList();

	
}

void init_Mappa(){
/*
 * Crea una display list per la mappa 2D del labirinto.
 * Le celle sono disegnate a met: solo il muro inferiore e quello sx
 * se presenti. Il muro superiore e il destro vengono disegnati dalle celle vicine
 * La Cella maze.n*maze.m  una cella fasulla, utilizzata per disegnare il muro superiore
 * e destro dell'intero labirinto
 */
	int i, x, y;

	LaMappa = glGenLists(1);
	glNewList(LaMappa, GL_COMPILE);
	glColor3f(1.0,1.0,1.0);
	for(i = 0; i<=maze.m*maze.n; i++){
			if(i == maze.m*maze.n){
				glBegin(GL_LINE_STRIP);
					glVertex2i(0, maze.m*10);
					glVertex2i(maze.n*10, maze.m*10);
					glVertex2i(maze.n*10, 0);
				glEnd();
			}
			else{
				x = i%maze.n * 10;
				y = i/maze.n * 10;
				switch(maze.pcella[i].passaggio){
					case 1:
					case 8:
					case 9:
						glPushMatrix();
						glTranslatef(x,y,0.0);
						glBegin(GL_LINE_STRIP);
							glVertex2i(10, 0);
							glVertex2i(0, 0);
							glVertex2i(0, 10);
						glEnd();
						glPopMatrix();
						break;
					case 2:
					case 3:
					case 10:
					case 11:
						glPushMatrix();
						glTranslatef(x,y,0.0);
						glBegin(GL_LINES);
							glVertex2i(10, 0);
							glVertex2i(0, 0);
						glEnd();
						glPopMatrix();
						break;
					case 4:
					case 5:
					case 12:
					case 13:
						glPushMatrix();
						glTranslatef(x,y,0.0);
						glBegin(GL_LINES);
							glVertex2i(0, 0);
							glVertex2i(0, 10);
						glEnd();
						glPopMatrix();
						break;
					case 6:
					case 7:
					case 14:
						break;
				}
			}
		}
	glEndList();
}

void init_Labirinto3D(){
/*
 * Crea una display list per i muri 3D del labirinto.
 * Il processo utilizzato per disegnare i muri di una singola cella
 *  lo stesso della mappa 2D.
 */

	int i, x, z;
	
	static GLfloat xtexture[] = {0.1, 0.0, 0.1, 0.0};
	static GLfloat ytexture[] = {0.0, 0.1, 0.0, 0.0};
	static GLfloat yptexture[] = {0.0, 1.0, 0.0, 0.0};
	
	//creazione del labirinto3D
	IlLabirinto = glGenLists(1);
	glNewList(IlLabirinto, GL_COMPILE);
		for(i = 0; i<maze.m*maze.n; i++){
			x = i%maze.n * 10;
			z = i/maze.n * 10;
			//printf("passaggio : %d\n",maze.pcella[i].passaggio);
			glBindTexture(GL_TEXTURE_2D, texture_id[0]);
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
			glTexGenfv(GL_S, GL_OBJECT_PLANE, xtexture);
			glTexGenfv(GL_T, GL_OBJECT_PLANE, ytexture);
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			switch(maze.pcella[i].passaggio){
				case 1:
					draw_1(x,z);
					break;
				case 2:
					draw_2(x,z);
					break;
				case 3:
					draw_3(x,z);
					break;
				case 4:
					draw_4(x,z);
					break;
				case 5:
					draw_5(x,z);
					break;
				case 6:
					draw_6(x,z);
					break;
				case 7:
					draw_7(x,z);
					break;
				case 8:
					draw_8(x,z);
					break;
				case 9:
					draw_9(x,z);
					break;
				case 10:
					draw_10(x,z);
					break;
				case 11:
					draw_11(x,z);
					break;
				case 12:
					draw_12(x,z);
					break;
				case 13:
					draw_13(x,z);
					break;
				case 14:
					draw_14(x,z);
					break;
			}
			glDisable(GL_TEXTURE_GEN_S);
			glDisable(GL_TEXTURE_GEN_T);
			glBindTexture(GL_TEXTURE_2D, texture_id[4]);
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
			glTexGenfv(GL_S, GL_OBJECT_PLANE, xtexture);
			glTexGenfv(GL_T, GL_OBJECT_PLANE, yptexture);
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			draw_pillar(x,z);
			draw_pillar(x,z+9);
			draw_pillar(x+9,z);
			draw_pillar(x+9,z+9);
			glDisable(GL_TEXTURE_GEN_S);
			glDisable(GL_TEXTURE_GEN_T);
		}
	glEndList();
}


short int caricaTexture(char *nomefile,GLubyte *ptr_texels)
{
/*
 * Funzione per caricare una bmp nel programma
 */
	unsigned short int id; /*identificatore file bmp*/

	unsigned int i,temp;

	if((fp=fopen(nomefile,"rb"))==NULL){
		printf("Errore lettura texture file %s",nomefile);
		return 0;
	}
	printf("\nApertura file %s in corso...",nomefile);
	if(fread(&id, sizeof(id), 1, fp) < 1){
	   printf("\nImpossibile leggere header file.");
	   fclose(fp);
	   return 0;
	}
	if(id != 0x4d42)	/* Controllo per la magic word BM al contrario... */
	{/* Non  una bitmap*/
		printf("\nNon e' un file bitmap.");
		fclose(fp);
		return 0;
	}

	fseek(fp,54L,SEEK_SET);
	if (fread(ptr_texels, 1,128*128*3, fp) < 1){
		//Impossibile leggere bitmap
		free(ptr_texels);
		fclose(fp);
		return 0;
	}

	/*Se nel file ho 145-45-0, vuol dire ke B=145, G=45 R=0.
	Con la gltexture2d invece assegno a R->145, G->45 e B->0. Ci  sbagliato!!
	Faccio lo swap tra la 1a e la 3a componente*/

	for(i=0;i<128*128*3;i=i+3){
		temp=ptr_texels[i];
		ptr_texels[i]=ptr_texels[i+2];
		ptr_texels[i+2]=temp;
	}   
	fclose(fp);
	return 1;
}

void draw_Pannel(float w, float h, int rows, int cols, int xp, int zyp, int quota){
/*
 * Disegna un quadrato sul piano xz come strip di quadrati
 * in maniera da ottenere un quadrato con mesh dipendente dalla densit
 * specificata in engine.h
 */

	int x, y;
        GLfloat vx, vy, s, t;
	GLfloat ts, tt, tw, th;
	
	ts = 1.0 / cols;
        tt = 1.0 / rows;
	
	tw = w /cols;
	th = h /rows;

	for(y = 0; y<rows; y++){
		glBegin(GL_QUAD_STRIP);
		for(x=0; x<=cols; x++){
			vx= tw*x;//-(w/2.0);
			vy= th*y;//-(h/2.0);
			s = ts*x;
			t = tt*y;

			glTexCoord2f(s, t);
			glColor3f(s, t, 0.0);
			glVertex3f(vx+xp, quota, vy+zyp);

			glColor3f(s, t +tt, 0.0);
			glTexCoord2f(s, t+tt);
			glVertex3f(vx+xp, quota, vy+th+zyp);
		}
		glEnd();
	}
}

void draw_1(int xp, int zyp){

	//muro verticale sx
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(0.9, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
}

void draw_2(int xp, int zyp){

	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 0.9);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_3(int xp, int zyp){

	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp+0.25);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 0.95);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_4(int xp, int zyp){

	//muro verticale sx
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(0.9, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_5(int xp, int zyp){

	//muro verticale sx
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
}

void draw_6(int xp, int zyp){

	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp-0.25, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(0.95, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_7(int xp, int zyp){

	//muro verticale dx
	glPushMatrix();
		glTranslatef(xp+9.5, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_8(int xp, int zyp){

	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale sx
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 0.9);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_9(int xp, int zyp){

	//muro verticale sx
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp+0.25, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(0.95, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();

}

void draw_10(int xp, int zyp){

	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_11(int xp, int zyp){

	//muro orizzontale sotto
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_12(int xp, int zyp){

	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp+0.25, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(0.95, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
	//muro verticale 
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
	
}

void draw_13(int xp, int zyp){

	//muro verticale sx
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.25, 5.0, 5.0);
			glScalef(0.05, 1.0, 1.0);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
}

void draw_14(int xp, int zyp){

	//muro orizzontale sopra
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp+9.5);
		glPushMatrix();
			glTranslatef(5.0, 5.0, 0.25);
			glScalef(1.0, 1.0, 0.05);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
}

void draw_pillar(int xp, int zyp){
	glPushMatrix();
		glTranslatef(xp, 0.0, zyp);
		glPushMatrix();
			glTranslatef(0.5, 5.0, 0.5);
			glScalef(0.1, 1.0, 0.1);
			glutSolidCube(10);
		glPopMatrix();
	glPopMatrix();
}
