/*************************************************************
*	PROGETTO LABIRINTO
*
* nome file: <geometria.c>
*
* Autori:
* Thomas Carrabs
* Roberto Pisoni
*
* Progetto realizzato per l'esame di Geometria Computazionale
* e di Informatica Grafica presso il Dipartimento di Scienze
* dell'Informazione dell'universit Statale di Milano
*
* Tutto il codice  distribuito sotto licenza Gnu/GPL
* http://www.gnu.org/copyleft/gpl.html
* una copia della licenza si trova nella cartella License del
* Progetto
*
*************************************************************/

#include "engine.h"
#include <math.h>

/*-----------------------------VARIABALI ESTERNE------------------------------*/
extern struct maze maze;		//struttura contenente il labirinto (vedi in labirinto.h)
extern int *soluz_lab;			//puntatore alla soluzione del labirinto (vedi in labirinto.c)
extern GLuint IlLabirinto;		//nome display list del labirinto 3D (vedi engine.c)
extern GLuint LaMappa;			//nome display list della mappa del labirinto (vedi engine.c)
extern GLuint LaSoluzione;		//nome display list della soluzione del labirinto (vedi engine.c)
extern GLuint IlPavimento;		//nome display list del pavimento del labirinto (vedi engine.c)
extern GLuint IlSoffitto;
extern GLuint texture_id[MAX_TEXTURES];
/*---------------------------END VARIABALI ESTERNE----------------------------*/


short int help_flag;

coordCella *pcoordCella;
coordQuad *pcoordQuad;
const float gra_to_rad = 0.0174532925f;  //fattore di conversione gradi->radianti

GLsizei wh,ww;			/*dimensione finestra principale*/
GLsizei whmap,wwmap;	/*dimensione finestra mappa*/
GLsizei cw, ch;			/*dimensione schermo*/
int id_entrata, idMap;
float x_entrata, z_entrata, x_cov, z_cov;
GLdouble fov,asp_ratio;	//field of vision, aspect ratio
GLenum visualizzazione; //visualizzazione labirinto: wireframe o solida

GLuint main_fin,prosp,lab3d,mappa,help;		//id per le 2 finestre + quella principale + quella della mappa
GLvoid *stile_font;				//tipo di font per il testo
GLuint prosp_fin_w = 140, prosp_fin_h = 80;	//dimensione finestra fov,aspect,wireframe	
GLuint help_fin_w = 140, help_fin_h = 340;	//dimensione finestra help
GLuint lab3d_fin_w = 440, lab3d_fin_h = 440;	//dimensione finestra labirinto3d	
GLfloat rotY;
GLfloat sceneroty;
GLfloat xCam, zCam, angoloCam;
float xvCam, zvCam, yvCam;
GLfloat xpos, zpos;
GLfloat lookupdown;
short int flag;

time_t start_time;	//variabile per cronometrare il tempo


//GEOMETRIA() -> IL MAIN DELLA SITUAZIONE

void geometria(int argc, char *argv[]){

	cw = 1024;		//dimensione finestra sistema
	ch = 768;
	ww = wwmap = 640;	//dimensione iniziale finestre principale e mappa
	wh = whmap = 480; 
	lookupdown = 0.0f;
	fov=45.0;			//field of vision di 45 gradi
	asp_ratio=(float)ww/wh;		//aspect ratio pari a 4/3
	visualizzazione = GL_FILL;	//visualizzazione solida labirinto
	mappa = 0;					//la finestra della mappa inizialmente non esiste
	x_entrata = ((id_entrata%maze.n)*maze.l)+(maze.l/2);	
	z_entrata = ((id_entrata/maze.n)*maze.l)+(maze.l/2);
	
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE| GLUT_RGB | GLUT_DEPTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glutInitWindowSize(ww,wh);
	glutInitWindowPosition(10,10);

	/* Crea la finestra principale del programma */
	main_fin=glutCreateWindow("Labirinto 30L");
	glutReshapeFunc(main_fin_reshape);
	glutDisplayFunc(main_fin_display);
	/* Registra la funzione che gestisce gli input da tastiera */
	glutKeyboardFunc(main_fin_tastiera);

	/* Crea la sottofinestra dove viene visualizzato il labirinto 3D*/
	lab3d = glutCreateSubWindow(main_fin, GAP+prosp_fin_w+GAP, GAP, lab3d_fin_w, lab3d_fin_w);
	glutReshapeFunc(lab3d_reshape);
	glutDisplayFunc(lab3d_display);
	glutKeyboardFunc(main_fin_tastiera);
	
	/* Crea la sottofinestra dove sono visualizzate le info */
	prosp = glutCreateSubWindow(main_fin, GAP, GAP, prosp_fin_w, prosp_fin_h);
	glutReshapeFunc(prosp_reshape);
	glutDisplayFunc(prosp_display);

	/* Crea la sottofinestra dove sono visualizzate le informazioni di aiuto */
	help = glutCreateSubWindow(main_fin, GAP, GAP+prosp_fin_h+GAP, help_fin_w, help_fin_h);
	glutReshapeFunc(help_reshape);
	glutDisplayFunc(help_display);
	
	glutSetWindow(lab3d);
	
	glEnable(GL_TEXTURE_2D);
	
	/*	Inizializzazione */
	if(!init_Texture())	//funzione per inizializzare le texture
		exit(0);
	
	init_Pavimento();
	init_Soffitto();
	init_Labirinto3D();
	
	/*	MAIN LOOP	*/
	glutMainLoop();
}

void main_fin_reshape(int w, int h)
{
/*
 * Gestisce la modifica delle dimensioni della finestra principale
 * del programma
 */
	if(w<=0 || h<=0)
		w=h=1;
	ww=w;
	wh=h;
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, w, 0, h);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
    
	lab3d_fin_w = (w-GAP*3)/1.318;  //(640-60)/?=440 -> ? = 1.318
	lab3d_fin_h = (h-GAP*2)/1;		//(480-40)/?=440 -> ? = 1
	
	prosp_fin_w = (w-GAP*3)/4.143;  //(640-60)/?=140 -> ? = 4.143
	prosp_fin_h = (h-GAP*3)/5.25;	//(480-60)/?=80 -> ? = 5.25

	help_fin_w = (w-GAP*3)/4.143;	
	help_fin_h = (h-GAP*3)/1.235;	//(480-60)/?=340 -> ? = 1.235

	glutSetWindow(lab3d);
	glutPositionWindow(GAP+prosp_fin_w+GAP, GAP);
	glutReshapeWindow(lab3d_fin_w, lab3d_fin_h);
	glutSetWindow(prosp);
	glutPositionWindow(GAP, GAP);
	glutReshapeWindow(prosp_fin_w, prosp_fin_h);
	glutSetWindow(help);
	glutPositionWindow(GAP,GAP+prosp_fin_h+GAP);
	glutReshapeWindow(help_fin_w, help_fin_h);
}

void main_fin_display(void)
{
/*
 * Disegna la finestra principale del programma
 */
	glClearColor(0.8, 0.8, 0.8, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glColor3ub(0, 0, 0);
	stile_font=GLUT_BITMAP_HELVETICA_12;
	stampastringa(GAP, wh-GAP+3, "Parametri proiezione");
	stampastringa(GAP, wh-(prosp_fin_h+2*GAP-3), "Comandi");
	stampastringa(GAP+prosp_fin_w+GAP, wh-GAP+3, "Labirinto");
	
	glutSwapBuffers();
}

void stampastringa(GLuint x, GLuint y, char* stringa)
{
/*
 * Stampa una stringa all'interno della finestra openGL
 */
	char *s;
	glRasterPos2i(x, y);	//setta la posizione su schermo
	for (s = stringa; *s; s++)	
        	glutBitmapCharacter(stile_font, *s);
}

void main_fin_tastiera(unsigned char key, int x, int y)
{
/*
 * Funzione che gestisce gli input da tastiera
 */
	float tmp_xpos, tmp_zpos;

	tmp_xpos = xpos;
	tmp_zpos = zpos;
	switch (key) {
	case 'w':
		/* muoversi in avanti */
		xpos -= (float) sin(rotY * gra_to_rad)*0.5f;
		zpos -= (float) cos(rotY * gra_to_rad)*0.5f;
		aggiornaTraslazione();
		if(collisione(idMap)){
		//se c' collisione ri-assegna tmp_xpos e tmp_zpos a xpos zpos
			xpos = tmp_xpos + (float) sin(rotY * gra_to_rad)*0.5f;
			zpos = tmp_zpos + (float) cos(rotY * gra_to_rad)*0.5f;
			aggiornaTraslazione();
		}
		glutPostRedisplay();
		break;
	case 's':
		/* muoversi indietro */
		xpos += (float) sin(rotY * gra_to_rad)*0.5f;
		zpos += (float) cos(rotY * gra_to_rad)*0.5f;
		aggiornaTraslazione();
		if(collisione(idMap)){
			xpos = tmp_xpos - (float) sin(rotY * gra_to_rad)*0.5f;
			zpos = tmp_zpos - (float) cos(rotY * gra_to_rad)*0.5f;
			aggiornaTraslazione();
		}
		glutPostRedisplay();
		break;
	case 'l':
		/* ruotare verso destra */
		if(rotY>360.0f)
			rotY=0;
		rotY += 2.5f;//1.0f;//0.6f;
		aggiornaRotazione();
		glutPostRedisplay();
		break;
	case 'j':
		/* ruotare verso sinistra*/
		if(rotY<-360.0f)
			rotY=0;
		rotY -= 2.5f;//1.0f;//0.6f;
		aggiornaRotazione();
		glutPostRedisplay();
		break;
	case 'i':
		/* guardare verso l'alto */
		if(lookupdown > -89.0f)
			lookupdown -= 1.0f;
		aggiornaRotazione();
		glutPostRedisplay();
		break;
	case 'k':
		/* guardare verso il basso*/
		if(lookupdown < 89.0f)
			lookupdown += 1.0f;
		aggiornaRotazione();
		glutPostRedisplay();
		break;
	case 'd':
		/* spostamento laterale verso destra */
		xpos -= (float) cos(rotY * gra_to_rad)*0.5f;
		zpos += (float) sin(rotY * gra_to_rad)*0.5f;
		aggiornaTraslazione();
		if(collisione(idMap)){
			xpos = tmp_xpos + (float) cos(rotY * gra_to_rad)*0.5f;
			zpos = tmp_zpos - (float) sin(rotY * gra_to_rad)*0.5f;
			aggiornaTraslazione();
		}
		glutPostRedisplay();
		break;
	case 'a':
		/* spostamento laterale verso sinistra */
		xpos += (float) cos(rotY * gra_to_rad)*0.5f;
		zpos -= (float) sin(rotY * gra_to_rad)*0.5f;
		aggiornaTraslazione();
		if(collisione(idMap)){
			xpos = tmp_xpos - (float) cos(rotY * gra_to_rad)*0.5f;
			zpos = tmp_zpos + (float) sin(rotY * gra_to_rad)*0.5f;
			aggiornaTraslazione();
		}
		glutPostRedisplay();
		break;
	case 'v':
		/* incrementa il valore del field of view*/
		fov=fov+5.0;
		if(fov>120.0)
			fov=45.0;
		break;
	case 'V':
		/* decrementa il valore del field of view*/
		fov=fov-5.0;
		if(fov<45.0)
			fov=120.0;
		break;
	case 'b':
		/* incrementa il valore dell'aspect ratio */
		asp_ratio=asp_ratio+(float) ww/wh;
		if(asp_ratio>12.0)
			asp_ratio=(float) ww/wh;
		break;
	case 'B':
		/* decrementa il valore dell'aspect ratio */
		asp_ratio=asp_ratio-(float) ww/wh;
		if(asp_ratio<1.0)
			asp_ratio=(float) 12;
		break;
	case 'n':
		/* switcha tra visualizzazione in wireframe o solida*/
		if(visualizzazione == GL_FILL)
			visualizzazione = GL_LINE;
		else
			visualizzazione = GL_FILL;
		break;
	case 'm':
		/* crea/distrugge una nuova finestra con la mappa 2D del labirinto */
		if(!mappa)
		{ //crea finestra mappa
				glutInitWindowSize(ww/2,wh/2);
				glutInitWindowPosition(655,270);
				mappa=glutCreateWindow("Mappa Labirinto");
				glutReshapeFunc(mappa_reshape);
				glutDisplayFunc(mappa_display);
				glutKeyboardFunc(main_fin_tastiera);
				init_Mappa();
				init_Soluzione();
		}
		else{		//distruggi finestra
			glDeleteLists(LaMappa,0);
			glutDestroyWindow(mappa);
			glutSetWindow(main_fin);
			mappa=0;
		}
		glutPostRedisplay();
		break;
	case 'h':
		/* abilita/disabilita la soluzione sulla mappa 2D */
		if(mappa){
			if(help_flag ==  0)
				help_flag = 1;
			else
				help_flag = 0;
		}
		break;
	case 'q':
	case 27: //tasto esc
		exit(0); //esce dal programma
    }
	redisplay_all();
}

void redisplay_all(void)
{
/*
 * Ridisegna tutte le finestre attive
 */
	glutSetWindow(prosp);		//setta prosp come finestra attuale
	prosp_reshape(prosp_fin_w, prosp_fin_h);
	glutPostRedisplay();
	glutSetWindow(help);
	help_reshape(help_fin_w, help_fin_h);
	glutPostRedisplay();
	if(mappa)
	{
		glutSetWindow(mappa);
		mappa_reshape(wwmap,whmap);
		glutPostRedisplay();
	}
	glutSetWindow(lab3d);
	lab3d_reshape(lab3d_fin_w, lab3d_fin_h);
	glutPostRedisplay();
}

void prosp_reshape(int w, int h)
{
/*
 * gestice la modifica delle dimensioni della sottofinestra in cui sono visualizzate le info
 */
	if(w<=0 || h<=0) //controllo per la riduzione a icona: se w=h=0 il prog fa una divisione per zero
		w=h=1;
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, w, 0, h);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void prosp_display(void)
{
/*
 * disegna la sottofinestra in cui sono visualizzate le info
 */
	char t[20];
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	stile_font=GLUT_BITMAP_HELVETICA_12;
	stampastringa(0,prosp_fin_h-12,"FIELD OF VISION: ");
	sprintf(t,"%f",fov);
	stampastringa(50,prosp_fin_h-24,t);
	stampastringa(0,prosp_fin_h-36,"ASPECT RATIO: ");
	sprintf(t,"%f",asp_ratio);	
	stampastringa(50,prosp_fin_h-48,t);
	stampastringa(0,prosp_fin_h-60,"WIREFRAME: ");
	if(visualizzazione == GL_FILL)
		stampastringa(50, prosp_fin_h-72,"solida");
	else
		stampastringa(50, prosp_fin_h-72,"wireframe");
	glutSwapBuffers();
}

void lab3d_reshape(int w, int h)
{
/*
 * gestisce la modifica delle dimensioni della sottofinestra in cui  visualizzato
 * il labirinto 3D
 */
	if(h<=0)
		h=1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov,asp_ratio,0.8,1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClearColor(0.8, 0.8, 0.8, 0.0);
}

void lab3d_display(void)
{
/*
 * Disegna la sottofinestra in cui  visualizzato il labirinto 3D
 * Gestisce la posizione della telecamera, del mondo
 * Le trasformazioni applicate al mondo
 * Il passaggio dei parametri ai programmi CG
 */
	int xMap,zMap;
	static int flag_start=0;
	double tempo_tot;
	char t[20];

	float x_uscita, z_uscita;
	
	GLfloat xtrans, ztrans;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	
	id_entrata = maze.entrata->id;
	x_entrata = ((id_entrata%maze.n)*maze.l)+(maze.l/2);
	z_entrata = ((id_entrata/maze.n)*maze.l)+(maze.l/2);
	
	x_uscita = ((maze.uscita->id%maze.n)*maze.l)+(maze.l/2);
	z_uscita = ((maze.uscita->id/maze.n)*maze.l)+(maze.l/2);

	xMap = zMap = 0; //inserita per evitare warning in compilazione, vengono assegnati qualche riga sotto
	if(z_entrata == maze.l/2){
		//sono sul bordo inferiore
		flag = 1;
		x_cov = x_entrata;
		z_cov = z_entrata + maze.l/2;
	}
	else{
		if(z_entrata == ((maze.m-1)*maze.l+maze.l/2)){
			// sono sul bordo superiore
			flag = 2;
			x_cov = x_entrata;
			z_cov = z_entrata - maze.l/2;
		}
		else{
			//ora controlla la x
			if(x_entrata == maze.l/2){
				// sono sul bordo sx
				flag = 3;
				x_cov = x_entrata + maze.l/2;
				z_cov = z_entrata;
			}
			else{
				// sono sul bordo dx
				flag = 4;
				x_cov = x_entrata - maze.l/2;
				z_cov = z_entrata;
			}
		}
	}
	if(flag_start==0){
	if(xCam>=x_entrata-3.0 && xCam<=x_entrata+3.0){		//controllo per vedere se sono sulla cella di ingresso
		if(zCam>=z_entrata-3.0 && zCam<=z_entrata+3.0){
			start_time=time(NULL);						//inizio a cronometrare il tempo
			flag_start=1;								//flag per indicare che devo cronometrare solo una volta	
		}
	}
	}

	
	if(xCam>=x_uscita-3.0 && xCam<=x_uscita+3.0){		//controllo per vedere se sono sulla cella di uscita
		if(zCam>=z_uscita-3.0 && zCam<=z_uscita+3.0){
		tempo_tot=difftime(time(NULL),start_time);		//calcolo quanto tempo ho impiegato
		glViewport(0, 0,lab3d_fin_w, lab3d_fin_h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluOrtho2D(0, lab3d_fin_w, 0, lab3d_fin_h);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		stile_font=GLUT_BITMAP_TIMES_ROMAN_24;
		stampastringa((lab3d_fin_w-144)/2, (wh-24)/2,"COMPLIMENTI!!");
		stampastringa((lab3d_fin_w-300)/2, (wh-72)/2,"HAI RISOLTO IL LABIRINTO!!");
		stampastringa((lab3d_fin_w-204)/2, (wh-120)/2,"Tempo impiegato: ");
		sprintf(t,"%.1f",tempo_tot);
		stampastringa(lab3d_fin_w/2, (wh-168)/2,t);
		stampastringa((lab3d_fin_w+100)/2, (wh-168)/2,"Sec");
		if(mappa){	//controllo se ho usato la mappa
			stampastringa((lab3d_fin_w-300)/2, (wh-216)/2,"...ma non usare la mappa!!");
		}
		glutSwapBuffers();
		_sleep(3000);
		exit(0);
		}
    }
	
	aggiornaTraslazione();
	aggiornaRotazione();
	
	xtrans = -xpos;
	ztrans = -zpos;
	sceneroty = 360.0f - rotY;

	switch(flag){
		case 1:
		//inf
			xMap = (int) (x_entrata + xtrans)/maze.l;
			zMap = (int) (z_entrata + ztrans)/maze.l;
			break;
		case 2:
		//sup
			xMap = (int) (x_entrata - xtrans)/maze.l;
			zMap = (int) (z_entrata - ztrans)/maze.l;
			break;
		case 3:
		//sx
			xMap = (int) (x_entrata + ztrans)/maze.l;
			zMap = (int) (z_entrata - xtrans)/maze.l;
			break;
		case 4: 
		//dx
			xMap = (int) (x_entrata - ztrans)/maze.l;
			zMap = (int) (z_entrata + xtrans)/maze.l;
			break;
	}

	idMap = zMap*maze.n + xMap;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glScalef(-1.0,1.0,1.0);
	glRotatef(lookupdown, 1.0f, 0.0f, 0.0f);
	glRotatef(sceneroty, 0.0f, 1.0f, 0.0f);
	
	glTranslatef(xtrans, 0, ztrans);
	gluLookAt(x_entrata,maze.l/2,z_entrata,x_cov,maze.l/2,z_cov,0.0,1.0,0.0);

	glPolygonMode(GL_FRONT_AND_BACK, visualizzazione);
	glColor3f(1.0, 0.0, 1.0);

	glCallList(IlPavimento);
	
	glCallList(IlSoffitto);
	
	glCallList(IlLabirinto);
	
	glPopMatrix();

	glutSwapBuffers();
}

void mappa_reshape(int w, int h)
{
/*
 * Gestisce la modifica delle dimensioni della finestra in cui  visualizzata la mappa2D
 */
	if(h<=0)
		h=1;
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, w, 0, h);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	wwmap=w;
	whmap=h;
	glClearColor(0.0, 0.0, 0.0, 0.0);
}

void mappa_display(void)
{
/*
 * Disegna la finestra in cui  visualizzata la mappa2D del labirinto
 */
	int i, x, y;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	

	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	//scala mappa labirinto per rappresentarla tutta
	glScalef((float)wwmap/(maze.l*maze.n),(float)whmap/(maze.l*maze.m),1.0);
	if(help_flag == 1)
		glCallList(LaSoluzione);

	
	//segnalino uscita
	i = maze.uscita->id;
	x = i%maze.n * 10;
	y = i/maze.n * 10;
	glPushMatrix();
		glTranslatef(x,y,0.0);
		glColor3f(0.0, 1.0, 0.0);
		glBegin(GL_QUADS);
			glVertex2i(0.0, 0.0);
			glVertex2i(10.0, 0.0);
			glVertex2i(10.0, 10.0);
			glVertex2i(0.0, 10.0);
		glEnd();
	glPopMatrix();
	//fine segnalino uscita
	//
	//segnalino entrata
	i = maze.entrata->id;
	x = i%maze.n * 10;
	y = i/maze.n * 10;
	glPushMatrix();
		glTranslatef(x,y, 0.0);
		glColor3f(1.0, 0.0, 0.0);
		glBegin(GL_QUADS);
			glVertex2i(0.0, 0.0);
			glVertex2i(10.0, 0.0);
			glVertex2i(10.0, 10.0);
			glVertex2i(0.0, 10.0);
		glEnd();
	glPopMatrix();
	//fine segnalino entrata
	//
	//segnalino posizione corrente
	x = idMap%maze.n * 10;
	y = idMap/maze.n * 10;
	glPushMatrix();
		glTranslatef(x,y, 0.0);
		glColor3f(0.0, 0.9, 1.0);
		glBegin(GL_QUADS);
			glVertex2i(0.0, 0.0);
			glVertex2i(10.0, 0.0);
			glVertex2i(10.0, 10.0);
			glVertex2i(0.0, 10.0);
		glEnd();
	glPopMatrix();
	
	/* DISEGNA LA MAPPA	*/
	glCallList(LaMappa);
	glPopMatrix();


	glutSwapBuffers();
}

void help_reshape(int w, int h)
{
/*
 * Gestisce la modifica delle dimensioni della sottfinestra di help
 */
	if(w<=0 || h<=0) //controllo per la riduzione a icona: se w=h=0 il prog fa una divisione per zero
		w=h=1;
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, w, 0, h);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void help_display(void)
{
/*
 * Disegna la sottofinestra di help
 */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glClearColor(1.0,1.0,1.0,0.0);
	glColor3f(0.0,0.0,0.0);
	stile_font=GLUT_BITMAP_HELVETICA_12;
	stampastringa(0,help_fin_h-12,"w: movimento avanti");
	stampastringa(0,help_fin_h-27,"s: movimento indietro");
	stampastringa(0,help_fin_h-42,"l: movimento destra");
	stampastringa(0,help_fin_h-57,"j: movimento sinistra");
	stampastringa(0,help_fin_h-72,"i: guarda alto");
	stampastringa(0,help_fin_h-87,"k:  guarda basso");
	stampastringa(0,help_fin_h-102,"d: movimento laterale dx");
	stampastringa(0,help_fin_h-117,"a: movimento laterale sx");
	stampastringa(0,help_fin_h-132,"v: aumenta FOV");
	stampastringa(0,help_fin_h-147,"V: riduci FOV");
	stampastringa(0,help_fin_h-162,"b: aumenta aspect ratio");
	stampastringa(0,help_fin_h-177,"B: riduci aspect ratio");
	stampastringa(0,help_fin_h-192,"n: wireframe/solido");
	stampastringa(0,help_fin_h-207,"m: mappa ON/OFF");
	stampastringa(0,help_fin_h-222,"h: percorso d'uscita");
	stampastringa(0,help_fin_h-237,"q - ESC: termina");
	glColor3f(1.0,1.0,1.0);
	glutSwapBuffers();
}

//void aggiornaCoordCameraReale() //DEPRECATED
//

void aggiornaTraslazione(){
/*
 * Aggiorna i valori di traslazione della telecamera
 */
	switch (flag){
		case 1:
			xCam = x_entrata - xpos;
			zCam = z_entrata - zpos;
			break;
		case 2:
			xCam = x_entrata + xpos;
			zCam = z_entrata + zpos;
			break;
		case 3:
			xCam = x_entrata - zpos;
			zCam = z_entrata + xpos;
			break;
		case 4:
			xCam = x_entrata + zpos;
			zCam = z_entrata - xpos;
			break;
	}
}



void aggiornaRotazione(){
/*
 * Aggiorna i valori di rotazione della telecamera
 */
	switch(flag){
		case 1:
			xvCam = sin(rotY * gra_to_rad) * cos(lookupdown * gra_to_rad);
			yvCam = -sin(lookupdown * gra_to_rad);
			zvCam = cos(rotY * gra_to_rad) * cos(lookupdown * gra_to_rad);
			break;
		case 2:
			xvCam = sin(rotY * gra_to_rad) * cos(lookupdown * gra_to_rad);
			yvCam = sin(lookupdown * gra_to_rad);
			zvCam = cos(rotY * gra_to_rad) * cos(lookupdown * gra_to_rad);
			break;
		case 3:
			angoloCam = 360.0f - rotY;
			zvCam = sin(angoloCam * gra_to_rad) * cos(lookupdown * gra_to_rad);
			yvCam = -sin(lookupdown * gra_to_rad);
			xvCam = cos(angoloCam * gra_to_rad) * cos(lookupdown * gra_to_rad);
			break;
		case 4:
			angoloCam = 360.0f - rotY;
			zvCam = sin(angoloCam * gra_to_rad) * cos(lookupdown * gra_to_rad);
			yvCam = sin(lookupdown * gra_to_rad);
			xvCam = cos(angoloCam * gra_to_rad) * cos(lookupdown * gra_to_rad);
			break;
	}
}


short int collisione(int ID){
/*
 * Semplice funzione che gestisce la collisione della telecamera con i muri
 * Visto che si tratta di verificare la collisione tra superfici piane e un 
 * singolo punto si  optato per il semplice confronto delle cordiante x e z.
 * la telecamera non pu infatti muoversi lungo l'asse y
 *
 * Il Confronto viene eseguito in base alla tipologia dell'attuale cella di labirnito
 * NORD,SUD,EST,OVEST indicano la presenza di un passaggio in quella direzione
 */
	float distanzaPiano;
	int x, z;

	switch(maze.pcella[ID].passaggio){
		case 1: //NORD
			distanzaPiano = (ID % maze.n)*maze.l;
			//muro sx
			if((xCam - distanzaPiano) <= 1.5)
				return (1);
			//muro dx
			distanzaPiano += maze.l;
			if(distanzaPiano - xCam <= 1.5)
				return (1);
			distanzaPiano = (ID / maze.n)*maze.l;
			//muro sotto
			if(zCam - distanzaPiano <= 1.5)
				return (1);
			break;
		case 2: //OVEST
			//muro sotto
			distanzaPiano = (ID / maze.n)*maze.l;
			if((zCam - distanzaPiano) <= 1.5 )
				return (1);
			//muro sopra
			distanzaPiano += maze.l;
			if(distanzaPiano - zCam <= 1.5)
				return (1);
			//muro destro
			distanzaPiano = (ID % maze.n)*maze.l+maze.l;
			if(distanzaPiano - xCam <= 1.5)
				return (1);
			break;
		case 3: //NORD_OVEST
			//muro sotto
			distanzaPiano = (ID / maze.n)*maze.l;
			if((zCam - distanzaPiano) <= 1.5 )
				return (1);
			//muro destro
			distanzaPiano = (ID % maze.n)*maze.l+maze.l;
			if(distanzaPiano - xCam <= 1.5)
				return (1);
			break;
		case 4: //SUD
			//muro sopra
			distanzaPiano = (ID/maze.n)*maze.l+maze.l;
			if((distanzaPiano - zCam) <= 1.5)
				return(1);
			//muro sx
			distanzaPiano = (ID%maze.n)*maze.l;
			if((xCam - distanzaPiano) <= 1.5)
				return(1);
			//muro dx
			distanzaPiano += maze.l;
			if((distanzaPiano - xCam) <= 1.5)
				return(1);
			break;
		case 5: //NORD-SUD
			//muro sx
			distanzaPiano = (ID%maze.n)*maze.l;
			if((xCam - distanzaPiano) <= 1.5)
				return(1);
			//muro dx
			distanzaPiano += maze.l;
			if((distanzaPiano - xCam) <= 1.5)
				return(1);
			break;
		case 6: //OVEST-SUD
			//muro sopra
			distanzaPiano = (ID/maze.n)*maze.l + maze.l;
			if((distanzaPiano - zCam) <= 1.5)
				return(1);
			//muro dx
			distanzaPiano = (ID%maze.n)*maze.l + maze.l;
			if((distanzaPiano - xCam) <= 1.5)
				return(1);
			break;
		case 7: //NORD-OVEST-SUD
			//muro dx
			distanzaPiano = (ID%maze.n)*maze.l + maze.l;
			if((distanzaPiano - xCam) <= 1.5)
				return(1);
			break;
		case 8: //EST
			//muro sotto
			distanzaPiano = (ID/maze.n)*maze.l;
			if((zCam - distanzaPiano) < 1.5)
				return(1);
			//muro sopra
			distanzaPiano += maze.l;
			if((distanzaPiano - zCam) <= 1.5)
				return(1);
			//muro sx
			distanzaPiano = (ID%maze.n)*maze.l;
			if((xCam - distanzaPiano) <= 1.5)
				return(1);
			break;
		case 9: //NORD-EST
			//muro sx
			distanzaPiano = (ID%maze.n)*maze.l;
			if((xCam - distanzaPiano) <= 1.5)
				return(1);
			//muro sotto
			distanzaPiano = (ID/maze.n)*maze.l;
			if((zCam - distanzaPiano) < 1.5)
				return(1);
			break;
		case 10: //EST-OVEST
			//muro sotto
			distanzaPiano = (ID/maze.n)*maze.l;
			if((zCam - distanzaPiano) < 1.5)
				return(1);
			//muro sopra
			distanzaPiano += maze.l;
			if((distanzaPiano - zCam) <= 1.5)
				return(1);
			break;
		case 11: //NORD-EST-OVEST
			//muro sotto
			distanzaPiano = (ID/maze.n)*maze.l;
			if((zCam - distanzaPiano) < 1.5)
				return(1);
			break;
		case 12: //EST-SUD
			//muro sx
			distanzaPiano = (ID%maze.n)*maze.l;
			if((xCam - distanzaPiano) <= 1.5)
				return(1);
			//muro sopra
			distanzaPiano = (ID/maze.n)*maze.l+maze.l;
			if((distanzaPiano - zCam) <= 1.5)
				return(1);
			break;
		case 13: //NORD-EST-SUD
			//muro sx
			distanzaPiano = (ID%maze.n)*maze.l;
			if((xCam - distanzaPiano) <= 1.5)
				return(1);
			break;
		case 14: //EST-SUD-OVEST
			//muro sopra
			distanzaPiano = (ID/maze.n)*maze.l+maze.l;
			if((distanzaPiano - zCam) <= 1.5)
				return(1);
			break;
	}

	//collisione con i pilastri
	//pilastro (x,z)
	x = (ID % maze.n)*maze.l;
	z = (ID / maze.n)*maze.l;
	if( (xCam-x)<=2 && (zCam-z)<=2 )
		return(1);
	//pilastro (x+9,z)
	x += 9;
	if( (x-xCam)<=2 && (zCam-z)<=2 )
		return(1);
	//pilastro (x,z+9)
	x -= 9;
	z += 9;
	if( (xCam-x)<=2 && (z-zCam)<=2 )
		return(1);
	//pilastro (x+9,z+9)
	x += 9;
	if( (x-xCam)<=2 && (z-zCam)<=2 )
		return(1);

	return (0);	
}

